@@ -470,11 +470,22 @@ type Hostinfo struct {
 	Services      []Service          `json:",omitempty"` // services advertised by this machine
 	NetInfo       *NetInfo           `json:",omitempty"`
 	SSH_HostKeys  []string           `json:"sshHostKeys,omitempty"` // if advertised
+	Cloud         string             `json:",omitempty"`
 
 	// NOTE: any new fields containing pointers in this type
 	//       require changes to Hostinfo.Equal.
 }
 
+// TailscaleSSHEnabled reports whether or not this node is acting as a
+// Tailscale SSH server.
+func (hi *Hostinfo) TailscaleSSHEnabled() bool {
+	// Currently, we use `SSH_HostKeys` as a proxy for this. However, we may later
+	// include non-Tailscale host keys, and will add a separate flag to rely on.
+	return hi != nil && len(hi.SSH_HostKeys) > 0
+}
+
+func (v HostinfoView) TailscaleSSHEnabled() bool { return v.ж.TailscaleSSHEnabled() }
+
 // NetInfo contains information about the host's network state.
 type NetInfo struct {
 	// MappingVariesByDestIP says whether the host's NAT mappings
